@@ -1,11 +1,8 @@
-"""
-    Slixmpp: The Slick XMPP Library
-    Copyright (C) 2011 Nathanael C. Fritz, Lance J.T. Stout
-    This file is part of Slixmpp.
-
-    See the file LICENSE for copying permission.
-"""
 
+# Slixmpp: The Slick XMPP Library
+# Copyright (C) 2011 Nathanael C. Fritz, Lance J.T. Stout
+# This file is part of Slixmpp.
+# See the file LICENSE for copying permission.
 import datetime as dt
 
 from slixmpp.jid import JID
@@ -18,7 +15,7 @@ class Delay(ElementBase):
     name = 'delay'
     namespace = 'urn:xmpp:delay'
     plugin_attrib = 'delay'
-    interfaces = set(('from', 'stamp', 'text'))
+    interfaces = {'from', 'stamp', 'text'}
 
     def get_from(self):
         from_ = self._get_attr('from')
