@@ -1,11 +1,8 @@
-"""
-    Slixmpp: The Slick XMPP Library
-    Copyright (C) 2010 Nathanael C. Fritz, Lance J.T. Stout
-    This file is part of Slixmpp.
-
-    See the file LICENSE for copying permission.
-"""
 
+# Slixmpp: The Slick XMPP Library
+# Copyright (C) 2010 Nathanael C. Fritz, Lance J.T. Stout
+# This file is part of Slixmpp.
+# See the file LICENSE for copying permission.
 import unittest
 from queue import Queue
 from xml.parsers.expat import ExpatError
@@ -20,9 +17,7 @@ from slixmpp.xmlstream.matcher import StanzaPath, MatcherId, MatchIDSender
 from slixmpp.xmlstream.matcher import MatchXMLMask, MatchXPath
 
 import asyncio
-cls = asyncio.get_event_loop().__class__
 
-cls.idle_call = lambda self, callback: callback()
 
 class SlixTest(unittest.TestCase):
 
@@ -228,6 +223,10 @@ class SlixTest(unittest.TestCase):
                     "Stanza:\n%s" % str(stanza))
         else:
             stanza_class = stanza.__class__
+            # Hack to preserve namespaces instead of having jabber:client
+            # everywhere.
+            old_ns = stanza_class.namespace
+            stanza_class.namespace =stanza.namespace
             if not isinstance(criteria, ElementBase):
                 xml = self.parse_xml(criteria)
             else:
@@ -235,8 +234,8 @@ class SlixTest(unittest.TestCase):
 
             # Ensure that top level namespaces are used, even if they
             # were not provided.
-            self.fix_namespaces(stanza.xml, 'jabber:client')
-            self.fix_namespaces(xml, 'jabber:client')
+            self.fix_namespaces(stanza.xml)
+            self.fix_namespaces(xml)
 
             stanza2 = stanza_class(xml=xml)
 
@@ -279,6 +278,7 @@ class SlixTest(unittest.TestCase):
                 debug += "Given stanza:\n%s\n" % highlight(tostring(stanza.xml))
                 debug += "Generated stanza:\n%s\n" % highlight(tostring(stanza2.xml))
                 result = self.compare(xml, stanza.xml, stanza2.xml)
+            stanza_class.namespace = old_ns
 
             self.assertTrue(result, debug)
 
@@ -332,6 +332,7 @@ class SlixTest(unittest.TestCase):
                                       plugin_config=plugin_config)
         else:
             raise ValueError("Unknown XMPP connection mode.")
+        self.xmpp._always_send_everything = True
 
         self.xmpp.connection_made(TestTransport(self.xmpp))
         self.xmpp.session_bind_event.set()
@@ -340,11 +341,19 @@ class SlixTest(unittest.TestCase):
         self.xmpp.default_lang = None
         self.xmpp.peer_default_lang = None
 
+        def new_id():
+            self.xmpp._id += 1
+            return str(self.xmpp._id)
+
+        self.xmpp._id = 0
+        self.xmpp.new_id = new_id
+
         # Must have the stream header ready for xmpp.process() to work.
         if not header:
             header = self.xmpp.stream_header
 
         self.xmpp.data_received(header)
+        self.wait_for_send_queue()
 
         if skip:
             self.xmpp.socket.next_sent()
@@ -361,6 +370,7 @@ class SlixTest(unittest.TestCase):
         # Some plugins require messages to have ID values. Set
         # this to True in tests related to those plugins.
         self.xmpp.use_message_ids = False
+        self.xmpp.use_presence_ids = False
 
     def make_header(self, sto='',
                           sfrom='',
@@ -423,7 +433,9 @@ class SlixTest(unittest.TestCase):
             timeout      -- Time to wait in seconds for data to be received by
                             a live connection.
         """
+        self.wait_()
         self.xmpp.data_received(data)
+        self.wait_()
 
     def recv_header(self, sto='',
                           sfrom='',
@@ -591,6 +603,7 @@ class SlixTest(unittest.TestCase):
                             'id', 'stanzapath', 'xpath', and 'mask'.
                             Defaults to the value of self.match_method.
         """
+        self.wait_for_send_queue()
         sent = self.xmpp.socket.next_sent(timeout)
         if data is None and sent is None:
             return
@@ -600,13 +613,31 @@ class SlixTest(unittest.TestCase):
             self.fail("No stanza was sent.")
 
         xml = self.parse_xml(sent)
-        self.fix_namespaces(xml, 'jabber:client')
-        sent = self.xmpp._build_stanza(xml, 'jabber:client')
+        self.fix_namespaces(xml)
+        sent = self.xmpp._build_stanza(xml)
         self.check(sent, data,
                    method=method,
                    defaults=defaults,
                    use_values=use_values)
 
+    def wait_for_send_queue(self):
+        loop = asyncio.get_event_loop()
+        future = asyncio.ensure_future(self.xmpp.run_filters(), loop=loop)
+        queue = self.xmpp.waiting_queue
+        loop.run_until_complete(queue.join())
+        future.cancel()
+
+    def wait_(self):
+        async def yield_some():
+            for i in range(100):
+                await asyncio.sleep(0)
+        loop = asyncio.get_event_loop()
+        loop.run_until_complete(yield_some())
+
+    def run_coro(self, coro):
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(coro)
+
     def stream_close(self):
         """
         Disconnect the dummy XMPP client.
@@ -623,7 +654,7 @@ class SlixTest(unittest.TestCase):
     # ------------------------------------------------------------------
     # XML Comparison and Cleanup
 
-    def fix_namespaces(self, xml, ns):
+    def fix_namespaces(self, xml, ns=None):
         """
         Assign a namespace to an element and any children that
         don't have a namespace.
@@ -632,6 +663,10 @@ class SlixTest(unittest.TestCase):
             xml -- The XML object to fix.
             ns  -- The namespace to add to the XML object.
         """
+        if ns is None:
+            ns = 'jabber:client'
+            if self.xmpp:
+                ns = self.xmpp.default_ns
         if xml.tag.startswith('{'):
             return
         xml.tag = '{%s}%s' % (ns, xml.tag)
